package ru.dbotthepony.kstarbound.defs.image

import com.google.gson.Gson
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.defs.DirectAssetReference
import ru.dbotthepony.kstarbound.util.PathStack

/**
 * Хранит данные (пару) вида "/example/animated.png" у которого, вероятнее всего, есть "/example/animated.frames"
 *
 * @see [AtlasConfiguration.Companion.get]
 */
data class ImageReference(
	val image: DirectAssetReference,
	val config: AtlasConfiguration,
) {
	class Adapter(private val atlasRegistry: () -> AtlasConfiguration.Registry) : TypeAdapterFactory {
		override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
			if (type.rawType == ImageReference::class.java) {
				return object : TypeAdapter<ImageReference>() {
					private val assets = gson.getAdapter(DirectAssetReference::class.java)

					override fun write(out: JsonWriter, value: ImageReference?) {
						out.value(value?.image?.fullPath)
					}

					override fun read(`in`: JsonReader): ImageReference? {
						if (`in`.peek() == JsonToken.NULL)
							return null

						val image = assets.read(`in`)

						if (image.path.contains(':'))
							throw JsonSyntaxException("Expected atlas/image reference, but got sprite reference: $image")

						return ImageReference(image, atlasRegistry.invoke().get(image.fullPath))
					}
				} as TypeAdapter<T>
			}

			return null
		}
	}
}
